@@ -159,6 +159,41 @@ def get_disk_identifier(dev):
     return disk_identifier[0]
 
 
+def get_uefi_disk_identifier(dev):
+    """Get the uuid from the disk being exposed by the ramdisk.
+
+    This uuid is appended to the pxe config which will then be set as the root
+    and load the bootx64.efi file using chainloader and boot the machine.
+    This is helpful in deployments to nodes with multiple disks.
+
+    https://wiki.gentoo.org/wiki/GRUB2/Chainloading
+
+    :param dev: Path for the already populated disk device.
+    :raises InstanceDeployFailure: Image is not UEFI bootable.
+    :returns: The UUID of the partition.
+    """
+    partition_id = None
+    try:
+        report, _ = utils.execute('fdisk', '-l', dev, run_as_root=True)
+    except processutils.ProcessExecutionError as e:
+        msg = _('Failed to find the partition on the disk %s ') % e
+        LOG.error(msg)
+        raise exception.InstanceDeployFailure(msg)
+    for line in report.splitlines():
+        if line.startswith(dev) and 'EFI System' in line:
+            vals = line.split()
+            partition_id = vals[0]
+    try:
+        lsblk_output, _ = utils.execute('lsblk', '-PbioUUID', partition_id,
+                                        run_as_root=True)
+        disk_identifier = lsblk_output.split("=")[1].strip()
+        disk_identifier = disk_identifier.strip('"')
+    except processutils.ProcessExecutionError as e:
+        raise exception.InstanceDeployFailure("Image is not UEFI bootable. "
+                                              "Error: %s " % e)
+    return disk_identifier
+
+
 def is_iscsi_device(dev, node_uuid):
     """check whether the device path belongs to an iscsi device. """
 
@@ -294,7 +329,6 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     if commit:
         # write to the disk
         dp.commit()
-        _trigger_device_rescan(dev)
     return part_dict
 
 
@@ -417,20 +451,7 @@ def destroy_disk_metadata(dev, node_uuid):
                 utils.execute('wipefs', '--all', dev,
                               run_as_root=True,
                               use_standard_locale=True)
-    # NOTE(TheJulia): sgdisk attempts to load and make sense of the
-    # partition tables in advance of wiping the partition data.
-    # This means when a CRC error is found, sgdisk fails before
-    # erasing partition data.
-    # This is the same bug as
-    # https://bugs.launchpad.net/ironic-python-agent/+bug/1737556
-    dd_device = 'of=%s' % dev
-    gpt_backup = get_dev_block_size(dev) - 33
-    dd_seek = 'seek=%i' % gpt_backup
-    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
-                  run_as_root=True, use_standard_locale=True)
-    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, 'count=33',
-                  dd_seek, run_as_root=True, use_standard_locale=True)
-    # Go ahead and let sgdisk run as well.
+
     utils.execute('sgdisk', '-Z', dev, run_as_root=True,
                   use_standard_locale=True)
 
@@ -810,45 +831,6 @@ def fix_gpt_partition(device, node_uuid):
         raise exception.InstanceDeployFailure(msg)
 
 
-def _trigger_device_rescan(device):
-    """Sync and trigger device rescan.
-
-    Disk partition performed via parted, when performed on a ramdisk
-    do not have to honor the fsync mechanism. In essence, fsync is used
-    on the file representing the block device, which falls to the kernel
-    filesystem layer to trigger a sync event. On a ramdisk using ramfs,
-    this is an explicit non-operation.
-
-    As a result of this, we need to trigger a system wide sync operation
-    which will trigger cache to flush to disk, after which partition changes
-    should be visible upon re-scan.
-
-    When ramdisks are not in use, this also helps ensure that data has
-    been safely flushed across the wire, such as on iscsi connections.
-
-    :param device: The block device containing paritions that is attempting
-                   to be updated.
-    """
-    # TODO(TheJulia): This helper method was broken out for re-use, and
-    # does not have an explicit test case for itself, and it should, but
-    # it's steps are fairly explicitly checked with mocks on execute in
-    # the partition code.
-    LOG.debug('Explicitly calling sync to force buffer/cache flush.')
-    utils.execute('sync')
-    # Make sure any additions to the partitioning are reflected in the
-    # kernel.
-    LOG.debug('Waiting until udev event queue is empty')
-    utils.execute('udevadm', 'settle')
-    try:
-        utils.execute('partprobe', device, run_as_root=True,
-                      attempts=CONF.disk_utils.partprobe_attempts)
-        # Also verify that the partitioning is correct now.
-        utils.execute('sgdisk', '-v', device, run_as_root=True)
-    except processutils.ProcessExecutionError as exc:
-        LOG.warning('Failed to verify partitioning after creating '
-                    'partition(s): %s', exc)
-
-
 def create_config_drive_partition(node_uuid, device, configdrive):
     """Create a partition for config drive
 
@@ -931,8 +913,25 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                 utils.execute('parted', '-a', 'optimal', '-s', '--', device,
                               'mkpart', 'primary', 'fat32', startlimit,
                               endlimit, run_as_root=True)
-            # Trigger device rescan
-            _trigger_device_rescan(device)
+            # Parted uses fsync to tell the kernel to sync file io
+            # however on ramdisks in ramfs, this is an explicit no-op.
+            # Explicitly call sync so when the the kernel attempts to read
+            # the partition table from disk, it is less likely that the write
+            # is still in buffer cache pending write to disk.
+            LOG.debug('Explicitly calling sync to force buffer/cache flush.')
+            utils.execute('sync')
+            # Make sure any additions to the partitioning are reflected in the
+            # kernel.
+            LOG.debug('Waiting until udev event queue is empty')
+            utils.execute('udevadm', 'settle')
+            try:
+                utils.execute('partprobe', device, run_as_root=True,
+                              attempts=CONF.disk_utils.partprobe_attempts)
+                # Also verify that the partitioning is correct now.
+                utils.execute('sgdisk', '-v', device, run_as_root=True)
+            except processutils.ProcessExecutionError as exc:
+                LOG.warning('Failed to verify GPT partitioning after creating '
+                            'the configdrive partition: %s', exc)
 
             upd_parts = set(part['number'] for part in list_partitions(device))
             new_part = set(upd_parts) - set(cur_parts)
