@@ -428,6 +428,68 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                                     conv_flags='sparse')
 
 
+class GetUEFIDiskIdentifierTestCase(base.IronicLibTestCase):
+
+    def setUp(self):
+        super(GetUEFIDiskIdentifierTestCase, self).setUp()
+        self.dev = '/dev/fake'
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_get_uefi_disk_identifier_uefi_bootable_image(self, mock_execute):
+        mock_execute.return_value = ('', '')
+        fdisk_output = """
+Disk /dev/sda: 931.5 GiB, 1000171331584 bytes, 1953459632 sectors
+Units: sectors of 1 * 512 = 512 bytes
+Sector size (logical/physical): 512 bytes / 512 bytes
+I/O size (minimum/optimal): 262144 bytes / 262144 bytes
+Disklabel type: gpt
+Disk identifier: 73457A6C-3595-4965-8D83-2EA1BD85F327
+
+Device          Start        End    Sectors   Size Type
+/dev/fake-part1        2048    1050623    1048576   512M EFI System
+/dev/fake-part2     1050624 1920172031 1919121408 915.1G Linux filesystem
+/dev/fake-part3  1920172032 1953458175   33286144  15.9G Linux swap
+"""
+        partition_id = '/dev/fake-part1'
+        lsblk_output = 'UUID="ABCD-B05B"\n'
+        part_result = 'ABCD-B05B'
+        mock_execute.side_effect = [(fdisk_output, ''), (lsblk_output, '')]
+        result = disk_utils.get_uefi_disk_identifier(self.dev)
+        self.assertEqual(part_result, result)
+        execute_calls = [
+            mock.call('fdisk', '-l', self.dev, run_as_root=True),
+            mock.call('lsblk', '-PbioUUID', partition_id,
+                      run_as_root=True)
+        ]
+        mock_execute.assert_has_calls(execute_calls)
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_get_uefi_disk_identifier_non_uefi_bootable_image(self,
+                                                              mock_execute):
+        mock_execute.return_value = ('', '')
+        fdisk_output = """
+Disk /dev/vda: 50 GiB, 53687091200 bytes, 104857600 sectors
+Units: sectors of 1 * 512 = 512 bytes
+Sector size (logical/physical): 512 bytes / 512 bytes
+I/O size (minimum/optimal): 512 bytes / 512 bytes
+Disklabel type: dos
+Disk identifier: 0xb82b9faf
+
+Device     Boot Start       End   Sectors Size Id Type
+/dev/fake-part1  *     2048 104857566 104855519  50G 83 Linux
+"""
+        partition_id = None
+        mock_execute.side_effect = [(fdisk_output, ''),
+                                    processutils.ProcessExecutionError()]
+        self.assertRaises(exception.InstanceDeployFailure,
+                          disk_utils.get_uefi_disk_identifier, self.dev)
+        execute_calls = [
+            mock.call('fdisk', '-l', self.dev, run_as_root=True),
+            mock.call('lsblk', '-PbioUUID', partition_id, run_as_root=True)
+        ]
+        mock_execute.assert_has_calls(execute_calls)
+
+
 @mock.patch.object(utils, 'execute', autospec=True)
 class MakePartitionsTestCase(base.IronicLibTestCase):
 
@@ -508,14 +570,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         fuser_cmd = ['fuser', 'fake-dev']
         fuser_call = mock.call(*fuser_cmd, run_as_root=True,
                                check_exit_code=[0, 1])
-
-        sync_calls = [mock.call('sync'),
-                      mock.call('udevadm', 'settle'),
-                      mock.call('partprobe', self.dev, attempts=10,
-                                run_as_root=True),
-                      mock.call('sgdisk', '-v', self.dev, run_as_root=True)]
-
-        mock_exc.assert_has_calls([parted_call, fuser_call] + sync_calls)
+        mock_exc.assert_has_calls([parted_call, fuser_call])
 
     def test_make_partitions(self, mock_exc):
         self._test_make_partitions(mock_exc, boot_option="netboot")
@@ -628,7 +683,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         self.assertEqual(expected_result, result)
 
 
-@mock.patch.object(utils, 'execute', autospec=True)
+@mock.patch.object(utils, 'execute', autospec=True, return_value=('', ''))
 class DestroyMetaDataTestCase(base.IronicLibTestCase):
 
     def setUp(self):
@@ -637,31 +692,9 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         self.node_uuid = "12345678-1234-1234-1234-1234567890abcxyz"
 
     def test_destroy_disk_metadata(self, mock_exec):
-        # Note(TheJulia): This list will get-reused, but only the second
-        # execution returning a string is needed for the test as otherwise
-        # command output is not used.
-        mock_exec.side_effect = iter([
-            (None, None),
-            ('1024\n', None),
-            (None, None),
-            (None, None),
-            (None, None),
-            (None, None)])
-
         expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True),
-                          mock.call('blockdev', '--getsz', 'fake-dev',
-                                    check_exit_code=[0],
-                                    run_as_root=True),
-                          mock.call('dd', 'bs=512', 'if=/dev/zero',
-                                    'of=fake-dev', 'count=33',
-                                    run_as_root=True,
-                                    use_standard_locale=True),
-                          mock.call('dd', 'bs=512', 'if=/dev/zero',
-                                    'of=fake-dev', 'count=33', 'seek=991',
-                                    run_as_root=True,
-                                    use_standard_locale=True),
                           mock.call('sgdisk', '-Z', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True),
@@ -687,26 +720,11 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True),
-                          mock.call('blockdev', '--getsz', 'fake-dev',
-                                    check_exit_code=[0],
-                                    run_as_root=True),
-                          mock.call('dd', 'bs=512', 'if=/dev/zero',
-                                    'of=fake-dev', 'count=33',
-                                    run_as_root=True,
-                                    use_standard_locale=True),
-                          mock.call('dd', 'bs=512', 'if=/dev/zero',
-                                    'of=fake-dev', 'count=33', 'seek=991',
-                                    run_as_root=True,
-                                    use_standard_locale=True),
                           mock.call('sgdisk', '-Z', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True)]
-        mock_exec.side_effect = iter([
-            (None, None),
-            ('1024\n', None),
-            (None, None),
-            (None, None),
-            processutils.ProcessExecutionError()])
+        mock_exec.side_effect = [(None, None),
+                                 processutils.ProcessExecutionError()]
         self.assertRaises(processutils.ProcessExecutionError,
                           disk_utils.destroy_disk_metadata,
                           self.dev,
@@ -714,14 +732,11 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         mock_exec.assert_has_calls(expected_calls)
 
     def test_destroy_disk_metadata_wipefs_not_support_force(self, mock_exec):
-        mock_exec.side_effect = iter([
-            processutils.ProcessExecutionError(description='--force'),
-            (None, None),
-            ('1024\n', None),
-            (None, None),
-            (None, None),
-            (None, None),
-            (None, None)])
+        mock_exec.side_effect = iter(
+            [processutils.ProcessExecutionError(description='--force'),
+             (None, None),
+             (None, None),
+             ('', '')])
 
         expected_call = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                    run_as_root=True,
@@ -777,9 +792,6 @@ class PopulateImageTestCase(base.IronicLibTestCase):
         self.assertFalse(mock_dd.called)
 
 
-# NOTE(TheJulia): _trigger_device_rescan is systemwide thus pointless
-# to execute in the file test case. Also, CI unit test jobs lack sgdisk.
-@mock.patch.object(disk_utils, '_trigger_device_rescan', lambda *_: None)
 @mock.patch.object(utils, 'wait_for_disk_to_become_available', lambda *_: None)
 @mock.patch.object(disk_utils, 'is_block_device', lambda d: True)
 @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
